/*
 * IFS.cpp
 *
 *  Created on: Jul 30, 2012
 *      Author: scottyoung
 */

#include "IFS.h"
#include <iostream>
#include <stdio.h>
#include <stdlib.h>


IFS::IFS(const char * filename) {
	FILE * specs;
	specs = fopen(filename, "r");
	int i = 0;

	if(specs != NULL && !feof(specs))
	{
		//First, read in the number of transformations
		fscanf(specs, "%d", &n);
		p = new float[n];
		transforms = new Matrix[n];
		while(!feof(specs) && i < n)
		{
			fscanf(specs, "%f", &p[i]);
			transforms[i].Read3x3(specs);
			i++;
		}
	}
}

IFS::~IFS() {
	if(transforms != NULL)
		delete[] transforms;
}

void IFS::render(Image img, int pts, int iters){
	Vec2f points[pts];
	int width = img.Width();
	int height = img.Height();

	//Initialize the points randomly
	for(int i=0; i<pts; i++)
		points[i].Set(drand48(), drand48());

	//Apply the matrix transformations to each point
	for(int i=0; i<iters; i++)
	{
		for(int a=0; a<pts; a++)
		{
			int t = randTransform();
			transforms[t].Transform(points[a]);
		}
	}

	//Paint the dots to the screen
	Vec3f black(0,0,0), white(1,1,1);
	img.SetAllPixels(black);
	for(int a=0; a<pts; a++)
	{
		int x = points[a].x()*width, y = points[a].y()*height;
		if(x>=0 && y>=0 && x<width && y<height)
			img.SetPixel(x, y, white);
	}

}

int IFS::randTransform()
{
	double pos = drand48();
	double cum = 0;
	for(int i=0; i<n; i++)
	{
		cum += p[i];
		if(cum > pos)
			return(i);
	}
	return(n-1);
}

