/*
 * main.cpp
 *
 *  Created on: Aug 10, 2012
 *      Author: scottyoung
 */

#import "scene_parser.h"
#import "hit.h"
#import "image.h"
#import "math.h"

Image renderColor(int resolution, SceneParser theScene, float t_min)
{
	Image img(resolution, resolution);
	Camera *cam = theScene.getCamera();
	Group *group = theScene.getGroup();
	Vec3f background = theScene.getBackgroundColor();

	img.SetAllPixels(background);
	for(int x=0; x<resolution; x++)
	{
		for(int y=0; y<resolution; y++)
		{
			Vec2f pos((float)x/resolution, (float)y/resolution);
			Ray r = cam->generateRay(pos);
			Hit h;
			if(group->intersect(r,h,t_min))
			{
				img.SetPixel(x, y, h.getMaterial()->color);
			}
		}
	}
	return(img);
}

Image renderDepth(int resolution, SceneParser theScene, float t_min, float d_lo, float d_hi)
{
	Image img(resolution, resolution);
	Camera *cam = theScene.getCamera();
	Group *group = theScene.getGroup();
	Vec3f background(0,0,0);

	img.SetAllPixels(background);
	for(int x=0; x<resolution; x++)
	{
		for(int y=0; y<resolution; y++)
		{
			Vec2f pos((float)x/resolution, (float)y/resolution);
			Ray r = cam->generateRay(pos);
			Hit h;
			if(group->intersect(r,h,t_min) && h.getT()<d_hi && h.getT()>d_lo)
			{
				float depth = 1.0 - (h.getT()-d_lo)/(d_hi-d_lo);
				Vec3f grey(depth, depth, depth);
				img.SetPixel(x, y, grey);
			}
		}
	}
	return(img);
}

int main()
{
	SceneParser theScene("scene4.txt");
	Group *theGroup = theScene.getGroup();
	Camera *theCamera = theScene.getCamera();
	Vec3f background = theScene.getBackgroundColor();

	theCamera->print();
	cout << "Background " << "R: " << background.r() << ", G: " << background.g()
			<< ", B: " << background.b() << endl;
	theGroup->print();

	Image scene = renderDepth(512, theScene, -INFINITY, 12, 17);
	scene.SaveTGA("scene4_depth.tga");

	return(0);
}



