/*
 * Sphere.cpp
 *
 *  Created on: Aug 10, 2012
 *      Author: scottyoung
 */

#include "Sphere.h"
#include <math.h>

Sphere::Sphere(Vec3f c, float r, Material *m) {
	center = c;
	radius = r;
	material = m;
}

Sphere::~Sphere() {

}

bool Sphere::intersect(const Ray &r, Hit &h, float tMin)
{
	bool hit = false;

	Vec3f E = r.getOrigin();
	Vec3f R = E - center;
	Vec3f D = r.getDirection();
	Vec3f N;

	//R.Write(); cout << " "; D.Write();

	float a = D.Dot3(D); //in transformed objects, dir may not be normalized
	float b = R.Dot3(D);
	float c = R.x()*R.x() + R.y()*R.y() + R.z()*R.z() - radius*radius;

	float discriminant = b*b-a*c;
	float hit1, hit2, tValue;

	/*cout << endl << "Testing Sphere Intersection " << endl; //DEBUG
	cout << "Discriminant: " << discriminant << endl;*/

	if(discriminant >= 0)
	{
		//Find the closest intersection
		hit1 = (-b  +  sqrt(discriminant)) / a;
		hit2 = (-b  -  sqrt(discriminant)) / a;

		if(hit2 > tMin && hit1 > tMin)
			tValue = min(hit1, hit2);
		else if(hit2 > tMin)
			tValue = hit2;
		else
			tValue = hit1;

		/*cout << "Hit 1: " << hit1 << endl;//DEBUG
		cout << "Hit 2: " << hit2 << endl;
		cout << "TVal:  " << tValue << endl;
		cout << "TMin:  " << tMin << endl;
		cout << "Current Best T: " << h.getT() << endl << endl;*/

		if(tValue > tMin && tValue < h.getT())
		{
			//Find the normal
			N = R+D*tValue;
			N.Normalize();
			h.set(tValue, material, N, r);
			hit = true;
		}
	}

	return(hit);
}

void Sphere::print(){
	cout << "Sphere: pos @(" << center.x() <<","<< center.y()<<","<<center.z()<<") radius = "<< radius << endl;
}

