/*
 * Plane.cpp
 *
 *  Created on: Aug 14, 2012
 *      Author: scottyoung
 */

#include "Plane.h"

#define EPSILON .0001

Plane::Plane(Vec3f n, float o, Material *m) {
	normal = n;
	normal.Normalize();
	offset = o;
	material = m;
	isGroup=false;
	isTri=false;
}

Plane::~Plane() {

}

bool Plane::intersect(const Ray &r, Hit &h, float tMin)
{
	bool isHit = false;
	Vec3f N = normal;
	Vec3f P = r.getOrigin();
	Vec3f D = r.getDirection();
	float a = offset;

	//Does (N * D) != 0? (we have a hit)
	if(fabs(N.Dot3(D)) > .0001)
	{
		float tValue = (a - N.Dot3(P))/(N.Dot3(D));
		if(tValue > tMin && tValue < h.getT())
		{
			h.set(tValue, material, N, r);
			isHit = true;
		}
	}

	RayTracingStats::IncrementNumIntersections();

	return(isHit);
}

void Plane::setBounds(){
	//only if the plane is perfectly axis-aligned does the bounding box have non-infinite parameters
	float xBounds = INFINITY, yBounds = INFINITY, zBounds = INFINITY;
	if(fabs(normal.x())-1.0 < EPSILON)
		xBounds = 0;
	else if(fabs(normal.y())-1.0 < EPSILON)
		yBounds = 0;
	else if(fabs(normal.z())-1.0 < EPSILON)
		zBounds = 0;
	bounds = BoundingBox(Vec3f(-xBounds,-yBounds,-zBounds), Vec3f(xBounds,yBounds,zBounds));
}

void Plane::print(){
	cout << "Plane: offset =" << offset <<" normal = "
			<< normal.x() <<","<< normal.y()<<","<<normal.z() << endl;
}

