/*
 * Triangle.cpp
 *
 *  Created on: Aug 14, 2012
 *      Author: scottyoung
 */

#include "Triangle.h"

Triangle::Triangle(Vec3f a, Vec3f b, Vec3f c, Material *m) {
	verts[0] = a;
	verts[1] = b;
	verts[2] = c;
	Vec3f::Cross3(normal, (b-a), (c-b));
	normal.Normalize();
	material = m;
	isGroup=false;
	isTri=true;
}

Triangle::~Triangle() {
	// TODO Auto-generated destructor stub
}

bool Triangle::intersect(const Ray &r, Hit &h, float tMin)
{
	bool isHit = false;
	float Beta;
	float Gamma;
	float tValue;
	float M;
	float A = verts[0].x()-verts[1].x();
	float B = verts[0].y()-verts[1].y();
	float C = verts[0].z()-verts[1].z();
	float D = verts[0].x()-verts[2].x();
	float E = verts[0].y()-verts[2].y();
	float F = verts[0].z()-verts[2].z();
	float G = r.getDirection().x();
	float H = r.getDirection().y();
	float I = r.getDirection().z();
	float J = verts[0].x()-r.getOrigin().x();
	float K = verts[0].y()-r.getOrigin().y();
	float L = verts[0].z()-r.getOrigin().z();

	M = A*(E*I - H*F) + B*(G*F - D*I) + C*(D*H - E*G);

	Beta = (J*(E*I-H*F) + K*(G*F - D*I) + L*(D*H - E*G))/M;
	Gamma = (I*(A*K - J*B) + H*(J*C - A*L) +G*(B*L - K*C))/M;

	if(Beta > 0 && Gamma > 0 && Beta+Gamma < 1)
	{
		tValue = -(F*(A*K - J*B) + E*(J*C - A*L) + D*(B*L - K*C))/M;
		if(tValue > tMin && tValue < h.getT())
		{
			isHit = true;
			h.set(tValue, material, normal, r);
		}

	}

	RayTracingStats::IncrementNumIntersections();

	return(isHit);
}

void Triangle::setBounds(){
	Vec3f lo( min(verts[0].x(), min(verts[1].x(), verts[2].x())),
			  min(verts[0].y(), min(verts[1].y(), verts[2].y())),
			  min(verts[0].z(), min(verts[1].z(), verts[2].z())));
	Vec3f hi( max(verts[0].x(), max(verts[1].x(), verts[2].x())),
			  max(verts[0].y(), max(verts[1].y(), verts[2].y())),
			  max(verts[0].z(), max(verts[1].z(), verts[2].z())));
	bounds = BoundingBox(lo, hi);
}

void Triangle::print(){
	cout << "Triangle: V1=(" << verts[0].x() <<","<< verts[0].y()<<","<<verts[0].z() << ") V2=("
			<< verts[1].x() <<","<< verts[1].y()<<","<<verts[1].z() << ") V3=("
			<< verts[2].x() <<","<< verts[2].y()<<","<<verts[2].z() << endl;
}

