#ifndef _SceneParser_H_
#define _SceneParser_H_

#include "vectors.h"
#include "Group.h"
#include "OrthographicCamera.h"
#include "PerspectiveCamera.h"
#include "Sphere.h"
#include "Plane.h"
#include "light.h"
#include "Triangle.h"
#include "Transform.h"
#include "Grid.h"

#define MAX_PARSER_TOKEN_LENGTH 100

// ====================================================================
// ====================================================================

class SceneParser {

public:

  // CONSTRUCTORS, DESTRUCTOR & INITIALIZE
  SceneParser();
  SceneParser(const char *filename);
  ~SceneParser();

  // ACCESSORS
  Group* getGroup() { return group; }
  Camera* getCamera() { return camera; }
  Vec3f getBackgroundColor() { return background_color; }
  Light** getLights(){return lights;}
  int getNumLights(){return num_lights;}
  Material** getMaterials(){return materials;}
  int getNumMaterials(){return numMaterials;}
  Vec3f getAmbientLight(){return ambient_light;}
  int getToken(char token[MAX_PARSER_TOKEN_LENGTH]);
  void setGroup(Group* g){group = g;}

private:

  // HELPER FUNCTIONS
  void initialize();
  void parseFile();
  Group* parseGroup();
  void parseLights();
  void parseOrthographicCamera();
  void parsePerspectiveCamera();
  void parseBackground();
  void parseMaterial();

  Object3D* parseObject(char token[MAX_PARSER_TOKEN_LENGTH]);
  Sphere* parseSphere();
  Triangle* parseTriangle();
  Plane* parsePlane();
  Group* parseTriangleMesh();
  Transform* parseTransform();
  Grid* parseGrid();

  Vec3f readVec3f();
  float readFloat();
  int readInt();
  
  // REPRESENTATION
  Group *group;
  Camera *camera;
  int num_lights;
  Light **lights;
  Vec3f background_color;
  Vec3f ambient_light;

  int numMaterials;
  Material **materials;
  int materialIndex;
  FILE *file;

};

// ====================================================================
// ====================================================================

#endif
