/*
 * BoundingBox.h
 *
 *  Created on: Sep 11, 2012
 *      Author: scottyoung
 */

#ifndef BOUNDINGBOX_H_
#define BOUNDINGBOX_H_

#include "ray.h"
#include <algorithm>
using namespace std;

class BoundingBox {
public:
	BoundingBox(){}
	BoundingBox(Vec3f l, Vec3f h)
	{ lo = l; hi = h; }
	virtual ~BoundingBox(){}
	bool intersect(const Ray &r, float &tVal, float tMin = -INFINITY)
	{
		Vec3f dir = r.getDirection();
		Vec3f org = r.getOrigin();
		Vec3f t1( min((lo.x()-org.x())/dir.x(), (hi.x()-org.x())/dir.x()),
				  min((lo.y()-org.y())/dir.y(), (hi.y()-org.y())/dir.y()),
			      min((lo.z()-org.z())/dir.z(), (hi.z()-org.z())/dir.z()) );
		Vec3f t2( max((lo.x()-org.x())/dir.x(), (hi.x()-org.x())/dir.x()),
				  max((lo.y()-org.y())/dir.y(), (hi.y()-org.y())/dir.y()),
			      max((lo.z()-org.z())/dir.z(), (hi.z()-org.z())/dir.z()) );
		float tNear = max(t1.x(), max(t1.y(), t1.z()));
		float tFar = min(t2.x(), min(t2.y(), t2.z()));
		if(tNear <= tFar && tNear > tMin)
		{
			tVal = tNear;
			return(true);
		}
		return(false);
	}
	Vec3f getMin(){return lo;}
	Vec3f getMax(){return hi;}
	BoundingBox mergeBox(BoundingBox addition){
		Vec3f newLo(min(addition.lo.x(), lo.x()), min(addition.lo.y(), lo.y()), min(addition.lo.z(), lo.z()));
		Vec3f newHi(max(addition.hi.x(), hi.x()), max(addition.hi.y(), hi.y()), max(addition.hi.z(), hi.z()));
		return(BoundingBox(newLo, newHi));
	}
	void includePoint(Vec3f p){
		Vec3f newLo(min(p.x(), lo.x()), min(p.y(), lo.y()), min(p.z(), lo.z()));
		Vec3f newHi(max(p.x(), hi.x()), max(p.y(), hi.y()), max(p.z(), hi.z()));
		lo = newLo; hi = newHi;
	}
	//To be inside, a point must lie between lo/hi for all dimensions
	bool isInside(Vec3f pt){
		return (pt.x()>=lo.x() && pt.y()>=lo.y() && pt.z()>=lo.z() &&
				pt.x()<=hi.x() && pt.y()<=hi.y() && pt.z()<=hi.z());
	}
	//Boxes overlap if any of the corners of one box is inside this box
	bool isOverlapping(BoundingBox b){

		Vec3f p1(b.lo);							//mmm
		Vec3f p2(b.lo.x(), b.lo.y(), b.hi.z()); //mmM
		Vec3f p3(b.lo.x(), b.hi.y(), b.hi.z()); //mMM
		Vec3f p4(b.lo.x(), b.hi.y(), b.lo.z()); //mMm
		Vec3f p5(b.hi); 						//MMM
		Vec3f p6(b.hi.x(), b.hi.y(), b.lo.z()); //MMm
		Vec3f p7(b.hi.x(), b.lo.y(), b.lo.z()); //Mmm
		Vec3f p8(b.hi.x(), b.lo.y(), b.hi.z()); //MmM
		return(isInside(p1)||isInside(p2)||isInside(p3)||isInside(p4)||isInside(p5)||isInside(p6)||isInside(p7)||isInside(p8));
	}

private:
	Vec3f lo;
	Vec3f hi;
};

#endif /* BOUNDINGBOX_H_ */
