/*
 * OrthographicCamera.cpp
 *
 *  Created on: Aug 10, 2012
 *      Author: scottyoung
 */

#include "OrthographicCamera.h"

OrthographicCamera::OrthographicCamera(Vec3f c, Vec3f d, Vec3f u, float s) {
	center = c;
	direction = d;
	up = u;
	size = s;

	direction.Normalize();
	up.Normalize();
}

OrthographicCamera::~OrthographicCamera() {

}

Ray OrthographicCamera::generateRay(Vec2f point){

	Vec3f horizontal, origin;
	Vec3f::Cross3(horizontal, direction, up);
	origin = horizontal * (point.x()-.5)*size  +  up * (point.y()-.5)*size;
	origin += center;

	Ray r(direction, origin);

	return(r);
}

void OrthographicCamera::print(){
	cout << "Orthographic Camera: " << endl;
	cout << "\tCentered at: " << center.x() << ", " << center.y() << ", " << center.z() << endl;
	cout << "\tDirection : " << direction.x() << ", " << direction.y() << ", " << direction.z() << endl;
	cout << "\tUp: " << up.x() << ", " << up.y() << ", " << up.z() << endl;
	cout << "\tSize: " << size << endl;
}

// ====================================================================
// Create an orthographic camera with the appropriate dimensions that
// crops the screen in the narrowest dimension.
// ====================================================================

void OrthographicCamera::glInit(int w, int h)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (w > h)
		glOrtho(-size/2.0, size/2.0, -size*(float)h/(float)w/2.0, size*(float)h/(float)w/2.0, 0.5, 40.0);
	else
		glOrtho(-size*(float)w/(float)h/2.0, size*(float)w/(float)h/2.0, -size/2.0, size/2.0, 0.5, 40.0);
}

// ====================================================================
// Place an orthographic camera within an OpenGL scene
// ====================================================================

void OrthographicCamera::glPlaceCamera(void)
{
	gluLookAt(center.x(), center.y(), center.z(),
			center.x()+direction.x(), center.y()+direction.y(), center.z()+direction.z(),
			up.x(), up.y(), up.z());
}

// ====================================================================
// dollyCamera, truckCamera, and RotateCamera
//
// Asumptions:
//  - up is really up (i.e., it hasn't been changed
//    to point to "screen up")
//  - up and direction are normalized
// Special considerations:
//  - If your constructor precomputes any vectors for
//    use in 'generateRay', you will likely to recompute those
//    values at athe end of the these three routines
// ====================================================================

// ====================================================================
// dollyCamera: Move camera along the direction vector
// ====================================================================

void OrthographicCamera::dollyCamera(float dist)
{
	center += direction*dist;


}

// ====================================================================
// truckCamera: Translate camera perpendicular to the direction vector
// ====================================================================

void OrthographicCamera::truckCamera(float dx, float dy)
{
	Vec3f horizontal;
	Vec3f::Cross3(horizontal, direction, up);
	horizontal.Normalize();

	Vec3f screenUp;
	Vec3f::Cross3(screenUp, horizontal, direction);

	center += horizontal*dx + screenUp*dy;


}

// ====================================================================
// rotateCamera: Rotate around the up and horizontal vectors
// ====================================================================

void OrthographicCamera::rotateCamera(float rx, float ry)
{
	Vec3f horizontal;
	Vec3f::Cross3(horizontal, direction, up);
	horizontal.Normalize();

	// Don't let the model flip upside-down (There is a singularity
	// at the poles when 'up' and 'direction' are aligned)
	float tiltAngle = acos(up.Dot3(direction));
	if (tiltAngle-ry > 3.13)
		ry = tiltAngle - 3.13;
	else if (tiltAngle-ry < 0.01)
		ry = tiltAngle - 0.01;

	Matrix rotMat = Matrix::MakeAxisRotation(up, rx);
	rotMat *= Matrix::MakeAxisRotation(horizontal, ry);

	rotMat.Transform(center);
	rotMat.TransformDirection(direction);


}

