/*
 * PerspectiveCamera.cpp
 *
 *  Created on: Aug 14, 2012
 *      Author: scottyoung
 */

#include "PerspectiveCamera.h"
#include <math.h>
#define PI 3.141592

PerspectiveCamera::PerspectiveCamera(Vec3f c, Vec3f d, Vec3f u, float a) {
	center = c;
	direction = d;
	up = u;
	angle = a;

	direction.Normalize();
	up.Normalize();

}

PerspectiveCamera::~PerspectiveCamera() {
	// TODO Auto-generated destructor stub
}

Ray PerspectiveCamera::generateRay(Vec2f point){

	float yaw = (point.x()-.5) * angle;
	float pitch = (point.y()-.5) * angle;

	Vec3f horizontal;
	Vec3f::Cross3(horizontal, direction, up);

	double Alpha = sin(pitch);
	double Beta = sin(yaw);
	double Gamma = cos(angle/2.0);

	Vec3f dir = (direction*Gamma) + (horizontal*Beta) + (up*Alpha);

	dir.Normalize();

	Ray r(dir, center);
	return(r);
}

void PerspectiveCamera::print(){
	cout << "Perspective Camera: " << endl;
	cout << "\tCentered at: " << center.x() << ", " << center.y() << ", " << center.z() << endl;
	cout << "\tDirection : " << direction.x() << ", " << direction.y() << ", " << direction.z() << endl;
	cout << "\tUp: " << up.x() << ", " << up.y() << ", " << up.z() << endl;
	cout << "\tAngle: " << angle << endl;
}

// ====================================================================
// Create a perspective camera with the appropriate dimensions that
// crops or stretches in the x-dimension as necessary
// ====================================================================

void PerspectiveCamera::glInit(int w, int h)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(angle*180.0/3.14159, (float)w/float(h), 0.5, 40.0);
}

// ====================================================================
// Place a perspective camera within an OpenGL scene
// ====================================================================

void PerspectiveCamera::glPlaceCamera(void)
{
	gluLookAt(center.x(), center.y(), center.z(),
			center.x()+direction.x(), center.y()+direction.y(), center.z()+direction.z(),
			up.x(), up.y(), up.z());
}

// ====================================================================
// dollyCamera, truckCamera, and RotateCamera
//
// Asumptions:
//  - up is really up (i.e., it hasn't been changed
//    to point to "screen up")
//  - up and direction are normalized
// Special considerations:
//  - If your constructor precomputes any vectors for
//    use in 'generateRay', you will likely to recompute those
//    values at athe end of the these three routines
// ====================================================================

// ====================================================================
// dollyCamera: Move camera along the direction vector
// ====================================================================

void PerspectiveCamera::dollyCamera(float dist)
{
	center += direction*dist;


}

// ====================================================================
// truckCamera: Translate camera perpendicular to the direction vector
// ====================================================================

void PerspectiveCamera::truckCamera(float dx, float dy)
{
	Vec3f horizontal;
	Vec3f::Cross3(horizontal, direction, up);
	horizontal.Normalize();

	Vec3f screenUp;
	Vec3f::Cross3(screenUp, horizontal, direction);

	center += horizontal*dx + screenUp*dy;


}

// ====================================================================
// rotateCamera: Rotate around the up and horizontal vectors
// ====================================================================

void PerspectiveCamera::rotateCamera(float rx, float ry)
{
	Vec3f horizontal;
	Vec3f::Cross3(horizontal, direction, up);
	horizontal.Normalize();

	// Don't let the model flip upside-down (There is a singularity
	// at the poles when 'up' and 'direction' are aligned)
	float tiltAngle = acos(up.Dot3(direction));
	if (tiltAngle-ry > 3.13)
		ry = tiltAngle - 3.13;
	else if (tiltAngle-ry < 0.01)
		ry = tiltAngle - 0.01;

	Matrix rotMat = Matrix::MakeAxisRotation(up, rx);
	rotMat *= Matrix::MakeAxisRotation(horizontal, ry);

	rotMat.Transform(center);
	rotMat.TransformDirection(direction);
	direction.Normalize();


}

