/*
 * PhongMaterial.cpp
 *
 *  Created on: Sep 4, 2012
 *      Author: scottyoung
 */

#include "PhongMaterial.h"


PhongMaterial::~PhongMaterial() {
	// TODO Auto-generated destructor stub
}

Vec3f PhongMaterial::shade(const Ray &r, Hit *h, const Vec3f &dirToLight, const Vec3f &lightColor){
	Vec3f N = h->getNormal();

	//Normal diffuse lighting of Lambertian surface
	float d = dirToLight.Dot3(N);
	Vec3f cDiff(lightColor.r()*diffuseColor.r()*d,
			lightColor.g()*diffuseColor.g()*d,
			lightColor.b()*diffuseColor.b()*d);

	//get the reflection vector = -I + 2(I*N)N
	Vec3f I(dirToLight);
	Vec3f R = Mirror(I, N);
	Vec3f E(r.getDirection());
	E.Normalize();
	E.Negate();

	//Calculate the magnitude
	d = R.Dot3(E);
	Vec3f value;
	if(d>0)
	{
		d = pow(d, exponent);
		//Specular component
		Vec3f cSpec(lightColor.r()*specularColor.r()*d,
			lightColor.g()*specularColor.g()*d,
			lightColor.b()*specularColor.b()*d);
		value = cSpec + cDiff;
	}
	else
	{
		value = cDiff;
	}

	return(value);
}

// ====================================================================
// Set the OpenGL parameters to render with the given material
// attributes.
// ====================================================================

void PhongMaterial::glSetMaterial(void) const
{
  // Set the diffuse and ambient colors
  GLfloat diffuseArray[4];
  diffuseArray[0] = diffuseColor.x();
  diffuseArray[1] = diffuseColor.y();
  diffuseArray[2] = diffuseColor.z();
  diffuseArray[3] = 1.0;
  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, diffuseArray);

  // Set the specular color and exponent
  GLfloat specularArray[4];
  specularArray[0] = specularColor.x();
  specularArray[1] = specularColor.y();
  specularArray[2] = specularColor.z();
  specularArray[3] = 1.0;
  glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specularArray);

  // GL uses the Blinn Torrance version of Phong so we need to scale
  // the exponent so it looks similar (NOTE: This is *not* a correct
  // mapping of Phong to Blinn-Torrance!!  There will be visual
  // differences between the GL version and the raytraced version!)
  float glexponent = float(exponent * 4.0f);
  if (glexponent > 128) glexponent = 128;
  if (glexponent < 0) glexponent = 0;
  glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, &glexponent);
}


